module namespace appvars = "http://www.architextus.com/xquery/library/how2_appvars";
declare namespace rest="http://exquery.org/ns/restxq";

declare option db:chop 'no';

(: *************************************************************** :)

(: Cannot put in config.xml, need to know application location in order to access config :)
declare variable $appvars:url-base := db:open('AppResources')//*[@id='config']/descendant::this-url-base/node();
declare variable $appvars:staging-url-base := db:open('AppResources')//*[@id='config']/descendant::staging-url-base/node();
declare variable $appvars:public-url-base := db:open('AppResources')//*[@id='config']/descendant::public-url-base/node();

(: *************************************************************** :)

declare variable $appvars:basex_rest_path := concat($appvars:url-base, "rest/");
declare variable $appvars:basex_restxq_path := concat($appvars:url-base, "restxq/");

declare variable $appvars:app-resource-location := concat($appvars:basex_rest_path, 'AppResources/');
declare variable $appvars:rel-path-to-app-icons := 'AppResources/ScriptsAndStyles/icons';

declare variable $appvars:dtd-topic := concat($appvars:app-resource-location, 'dtds/HowToContent_Topics.dtd');
declare variable $appvars:dtd-menu := concat($appvars:app-resource-location, 'dtds/HowToContent_Toc.dtd');
declare variable $appvars:dtd-xliff := concat($appvars:app-resource-location, 'dtds/xliff-core-1.2-strict.xsd');
declare variable $appvars:add-headings-to-topicrefs-xsl := concat($appvars:app-resource-location, 'ScriptsAndStyles/AddHeadingsToMenuTopicrefs.xsl');

declare variable $appvars:basex_image_rest_path := concat($appvars:basex_rest_path, 'ImagesAndGlobalContent/');
declare variable $appvars:images-location := concat($appvars:basex_image_rest_path, 'images.xml');

declare variable $appvars:request-html-topic := concat($appvars:basex_restxq_path, 'how2-html-topic/');

(: Languages :)

declare variable $appvars:target-languages := "ar-eg bg-bg ca-es cs-cz da-dk de-de el-gr en-gb es-es es-us fi-fi fr-ca fr-fr hr-hr hu-hu it-it iw-il lt-lt lv-lv ja-jp ko-kr nb-no nl-nl pl-pl pt-br pt-pt ro-ro ru-ru sk-sk sl-sl sr-rs sv-se th-th tr-tr uk-ua vi-vn zh-cn zh-tw";

declare variable $appvars:all-languages := "ar-eg bg-bg ca-es cs-cz da-dk de-de el-gr en-gb en-us es-es es-us fi-fi fr-ca fr-fr hr-hr hu-hu it-it iw-il lt-lt lv-lv ja-jp ko-kr nb-no nl-nl pl-pl pt-br pt-pt ro-ro ru-ru sk-sk sl-sl sr-rs sv-se th-th tr-tr uk-ua vi-vn zh-cn zh-tw";

declare variable $appvars:rtl-languages := "ar-eg iw-il";

declare %rest:path("/how2/rtl-languages")
        %rest:GET
        function appvars:get-rtl-languages() as node(){
        let $languages := $appvars:rtl-languages
        return <rtl-languages>{$languages}</rtl-languages>
};

(: Default filters :)
declare variable $appvars:default_device := 'Razr';

(: ************************************************************************ :)
(: *** CSS **************************************************************** :)
(: ************************************************************************ :)

declare function appvars:get-css-dash() as xs:string{
  let $css := concat($appvars:app-resource-location, "ScriptsAndStyles/dash.css")  
  return $css
};

declare %rest:path("/how2/get-main-css")
   %rest:GET  
   function appvars:get-css-main-as-node() as node(){
   let $css := appvars:get-css-main-name()
   (: If css on outside server, config id it as path, otherwise it provides the name :)
   return if (starts-with ($css, 'http://')) 
          then <css>{$css}</css>
          else <css>{concat($appvars:basex_rest_path, 'AppResources/ScriptsAndStyles/', $css)}</css>
};

declare %rest:path("/how2/get-main-css-static")
   %rest:GET  
   function appvars:get-css-main-static-as-node() as node(){
   let $css := appvars:get-css-main-name()
   (: If css on outside server, config id it as path, otherwise it provides the name (and file is copied at css/ during output preparation) :)
   return if (starts-with ($css, 'http://')) 
          then <css>{$css}</css>
          else <css>{concat('css/', $css)}</css>
};

declare  function appvars:get-css-main-name() as xs:string{
   let $css := db:open("AppResources")/descendant::*[name()='css']/descendant::*[name()='html-main'] 
   return $css
};

declare  function appvars:get-css-review-name() as xs:string{
   let $css := db:open("AppResources")/descendant::*[name()='css']/descendant::*[name()='html-review'] 
   return $css
};

declare function appvars:get-css-main() as xs:string{
  let $css := if (starts-with( appvars:get-css-main-name(), 'http://'))
              then appvars:get-css-main-name()
              else concat($appvars:app-resource-location, "ScriptsAndStyles/", appvars:get-css-main-name())  
  return $css
};

declare function appvars:get-css-review() as xs:string{
  let $css := concat($appvars:app-resource-location, "ScriptsAndStyles/", appvars:get-css-review-name())  
  return $css
};

(: ************************************************************************ :)
(: Feedback :)
(: ************************************************************************ :)

declare %rest:path("/how2/get-feedback-switch")
   %rest:GET  
   function appvars:get-feedback-switch() as node(){
   let $feedback-switch := db:open('AppResources')/descendant-or-self::*[@id='config']/descendant-or-self::*[@id='config']/descendant-or-self::*[name()='user-feedback']
   return $feedback-switch
};

(: ************************************************************************ :)
(: *** Translatable image, prompt and translatable string references ****** :)
(: ************************************************************************ :)

declare function appvars:get-string-ref-file ($lang as xs:string) as xs:string{
    let $file := concat($appvars:basex_rest_path, $lang, '/string-references.xml')
    return $file
};

declare function appvars:get-string-ref-file-xsl ($lang as xs:string) as xs:string{
    let $file := concat($appvars:basex_rest_path, $lang, '/string-references-transforms.xml')
    return $file
};

declare function appvars:get-image-ref-file() as xs:string{
    let $file := $appvars:images-location
    return $file
};

declare function appvars:get-prompt-ref-file () as xs:string{
    let $file := concat($appvars:basex_image_rest_path, '/prompts.xml')
    return $file
};

declare function appvars:get-siteref-file () as xs:string{
    let $file := concat($appvars:basex_image_rest_path, '/siterefs.xml')
    return $file
};

declare function appvars:get-video-ref-file () as xs:string{
    let $file := concat($appvars:basex_image_rest_path, '/videos.xml')
    return $file
};

declare %rest:path("/get-string-for-transforms/{$lang}/{$name}")
   %rest:GET
    function appvars:get-string-for-transforms ($name as xs:string, $lang as xs:string) as node(){
    for $x in db:open($lang)/string-references[@id='stringreferencestransform'] 
    return $x//string[@name = $name]
};


(: ************************************************************************ :)
(: *** XLSTs ************************************************************** :)
(: ************************************************************************ :)

declare function appvars:get-topic2html-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/HowToTransform_Topic.xsl")
    return $file
};

declare function appvars:get-translation-preparation-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/Xliff_PrepareSourcesForTranslation.xsl")
    return $file
};

declare function appvars:get-translation-xliff-to-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/Xliff_To.xsl")
    return $file
};

declare function appvars:get-translation-xliff-skeleton-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/Xliff_To_Skeleton.xsl")
    return $file
};

declare function appvars:get-translation-xliff-from-skeleton-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/Xliff_From_Skeleton.xsl")
    return $file
};

declare  function appvars:get-xliff-config-file-uri() as xs:string{
   let $config-file := concat($appvars:app-resource-location, "ScriptsAndStyles/Xliff_Config_How2Dtds.xml")
   return $config-file
};

declare function appvars:get-fo-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/FoTransform.xsl")
    return $file
};

declare function appvars:get-menu2html-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/HowToTransform_Menu.xsl")
    return $file
};

declare function appvars:get-menu2htmlreview-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/HowToTransform_MenuReview.xsl")
    return $file
};

(:declare function appvars:get-prepare-ids-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/FoPrepareIds.xsl")
    return $file
};:)

(:declare function appvars:get-filter-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/FilterContent.xsl")
    return $file
};


declare function appvars:get-prompt-xsl() as xs:string{
    let $file := concat($appvars:app-resource-location, "ScriptsAndStyles/ResolvePrompts.xsl")
    return $file
};:)

(: ************************************************************************ :)
(: *** Databases ********************************************************** :)
(: ************************************************************************ :)

declare function appvars:open-db($lang as xs:string) as node()*{
    let $db := db:open($lang, "")
    return $db
};

declare function appvars:open-skeletons-db($lang as xs:string) as node()*{
    let $db := db:open("Translation-management", concat($lang, '/skeletons'))
    return $db
};

declare function appvars:open-xliff-units-db($lang as xs:string) as node()*{
    let $db := db:open("Translation-management", concat($lang, '/xliff-units'))
    return $db
};

declare function appvars:get-all-lang-as-dropbox() as node()*{
    let $options := for $x in tokenize($appvars:all-languages, '&#32;')
                    let $option := <option value="{$x}">{$x}</option>
                    return $option
      return <select name="lang">{$options}</select>
};


declare function appvars:get-all-lang-as-html-checkboxes() as node()*{
    for $x in tokenize($appvars:all-languages, '&#32;')
      let $box := <span class="line"><input name="lang" type="checkbox" value="{$x}"></input>{concat('&#160;', $x)}</span>
      return $box
};

declare function appvars:get-all-menus-as-dropdown($lang as xs:string, $all-option as xs:string) as node(){
                        <select name="menu-id">
                        {if ($all-option = 'yes') then <option value="All">All</option>
                                                 else ''
                        }
                        {   
                             for $x in appvars:open-db($lang)//*[name()='menu']
                             let $id := $x/data(@id)
                             let $title := $x/child::*[name()='title']
                             return <option value="{$id}">{concat($title, '&#32;(', $id, ')')}</option>
                        }
                       </select>
};

(: ************************************************************************ :)
(: *** Image resolution *************************************************** :)
(: ************************************************************************ :)

declare function appvars:get-default-image-resolution() as xs:string{
 let $config := db:open('AppResources')//*[name()='config'][@id = 'config']
 let $default-res := $config/descendant::*[name()='screen-resolutions']/data(@default-screen-res)
 return $default-res
};

declare function appvars:get-resolution-for-model($model as xs:string) as xs:string{

   let $default-res := appvars:get-default-image-resolution()
   
   let $config := db:open('AppResources')//*[name()='config'][@id = 'config']
   let $res := if (exists($config/descendant::*[name()='screen-resolution'][@model=$model]))
               then $config/descendant::*[name()='screen-resolution'][@model=$model]/data(@screen-res)
               else $default-res

   return $res
};

declare function appvars:retrieve-image-with-best-resolution($image-rest-url as xs:string, $model as xs:string) as xs:string{
    let $db-name := substring-before(substring-after($image-rest-url, '/rest/'), '/')
    let $all-files := db:list($db-name)
    (: content before extensions:)
    let $image-rel-path := substring-after($image-rest-url, concat('/rest/', $db-name, '/'))
    let $best-resolution := appvars:get-resolution-for-model($model)
    let $default-resolution := appvars:get-default-image-resolution()
    let $best-image := replace($image-rel-path, '--d-p-i--', $best-resolution)
    let $default-image := replace($image-rel-path, '--d-p-i--', $default-resolution)
       
    (: $best-image is image with best resolution :)
    let $result := if (count(index-of($all-files, $best-image)) > 0)
                   then $best-image
                   else if (count(index-of($all-files, $default-image)) > 0)
                        then $default-image
                        else replace($image-rel-path, '/--d-p-i--', '') (: Leave as is if best/default are not found, for backward compatible :)
    return $result
};


declare function appvars:retrieve-image-with-best-resolution-old($image-rest-url as xs:string, $model as xs:string) as xs:string{ 
    let $db-name := substring-before(substring-after($image-rest-url, '/rest/'), '/')
    let $all-files := db:list($db-name)
    let $url-pieces := tokenize($image-rest-url, '\.')
    let $image-url-before-file-extension := string-join($url-pieces[position() != last()], '.')
    let $ext := concat('.', $url-pieces[position() = last()])
    let $base-name := substring-after($image-url-before-file-extension, concat('/',$db-name,'/'))
    let $best-base-name := concat($base-name, '-', appvars:get-resolution-for-model($model), '.')
    let $default-base-name := concat($base-name, '-', appvars:get-default-image-resolution())
    (: $best-image is image with best resolution :)
    let $best-image := for $x in $all-files
                       return if (starts-with($x, $best-base-name))
                              then $x
                              else ()
    let $best-image := if (count($best-image) = 1) then $best-image
                       else if (count($best-image) > 1) then $best-image[1]
                       else replace($best-base-name, appvars:get-resolution-for-model($model), appvars:get-default-image-resolution())
    
    (: If the best image exists we keep it, otherwise we try for the default :)
    let $right-dpi-image := if (db:exists($db-name, $best-image)) then $best-image
                  else replace($best-image, appvars:get-resolution-for-model($model), appvars:get-default-image-resolution())

    let $image-or-default := if (ends-with($right-dpi-image, '.')) 
                             then for $x in $all-files
                                  return if (starts-with($x, $right-dpi-image))
                                         then $x
                                         else ()
                             else $right-dpi-image
    (: If best or default dpi is not found, use exact value that was passed :)
    let $best-image-or-default := 
                       if (count($image-or-default) = 1) 
                       then $image-or-default
                       else if (count($image-or-default) > 1) 
                            then $image-or-default[1]
                            else concat(substring-after($image-url-before-file-extension, concat($db-name, '/') ), $ext)

    return $best-image-or-default
};

(: ************************************************************************ :)
(: *** Google analytics code ********************************************** :)
(: ************************************************************************ :)
 
declare %rest:path("/google-analytics-scripts")
   %rest:GET
   function appvars:get-google-analytics-script() as node(){

    let $code := "var _gaq=_gaq||[];_gaq.push([&quot;_setAccount&quot;,&quot;UA-12060932-4&quot;]);_gaq.push([&quot;_trackPageview&quot;]);(function(){var ga=document.createElement(&quot;script&quot;);ga.type=&quot;text/javascript&quot;;ga.async=true;ga.src=(&quot;https:&quot;==document.location.protocol?&quot;https://ssl&quot;:&quot;http://www&quot;)+&quot;.google-analytics.com/ga.js&quot;;var s=document.getElementsByTagName(&quot;script&quot;)[0];s.parentNode.insertBefore(ga,s)})();"
    let $script := <script type="text/javascript">{$code}
                   </script>
 
    return $script
};

(: ************************************************************************ :)
(: *** Images ************************************************************* :)
(: ************************************************************************ :)

declare %rest:path("/dynamic-image-resolution")
 %rest:GET
 %rest:query-param("image-uri", "{$image-uri}", '')
 %rest:query-param("model", "{$model}", 'default')
 function appvars:get-link-to-dynamic-image($image-uri as xs:string, $model as xs:string) as node(){  

    let $found-image := appvars:retrieve-image-with-best-resolution($image-uri, $model)
    return <value>{concat($appvars:basex_image_rest_path, $found-image)}</value>
        
};

declare %rest:path("/static-image-resolution")
 %rest:GET
 %rest:query-param("image-uri", "{$image-uri}", '')
 %rest:query-param("model", "{$model}", 'default')
 function appvars:get-link-to-static-image($image-uri as xs:string, $model as xs:string) as node(){ 
    let $restful-image-uri := if (starts-with($image-uri, 'http://'))
                              then $image-uri
                              else concat($appvars:basex_image_rest_path, $image-uri)
    let $found-image := appvars:retrieve-image-with-best-resolution($restful-image-uri, $model)
    return <value>{$found-image}</value>      
};

declare %rest:path("/dynamic-icon-resolution")
 %rest:GET
 %rest:query-param("image-uri", "{$image-uri}", '')
 %rest:query-param("model", "{$model}", 'default')
 function appvars:get-link-to-dynamic-icon($image-uri as xs:string, $model as xs:string) as node(){  

    let $found-image := appvars:retrieve-image-with-best-resolution($image-uri, $model)

    return <value>{concat($appvars:app-resource-location, $found-image)}</value>
        
};

declare %rest:path("/static-icon-resolution")
 %rest:GET
 %rest:query-param("image-uri", "{$image-uri}", '')
 %rest:query-param("model", "{$model}", 'default')
 function appvars:get-link-to-static-icon($image-uri as xs:string, $model as xs:string) as node(){  
    let $found-image := appvars:retrieve-image-with-best-resolution($image-uri, $model)
    return <value>{substring-after($found-image, 'ScriptsAndStyles/')}</value>
        
};

(: To break import in import :)
   declare function appvars:xml-get-how2-topic($id as xs:string, $lang as xs:string) as node()*{
    for $x in appvars:open-db($lang)//*[self::media or self::topic][@id=$id]
    return $x
};

(: ************************************************************************ :)
(: *** Navigation ********************************************************* :)
(: ************************************************************************ :)

declare function appvars:redirect($redirect as xs:string) {
  <restxq:response>
    <http:response status="302">
      <http:header name="location" value="{ $redirect }"/>
    </http:response>
  </restxq:response>
};